import numpy as np
from sklearn.model_selection import train_test_split

from insurance_claim_model.config.core import config
from insurance_claim_model.pipeline import predict_pipe
from insurance_claim_model.processing.data_manager import load_dataset, save_pipeline


def run_training() -> None:
    data = load_dataset(file_name=config.app_config.training_data_file)
    X_train, X_test, y_train, y_test = train_test_split(
        data[config.model_config.features],
        data[config.model_config.target],
        test_size=config.model_config.test_size,
        random_state=config.model_config.random_state,
    )

    y_train = np.log(y_train)
    predict_pipe.fit(X_train, y_train)

    save_pipeline(pipeline=predict_pipe)


if __name__ == "__main__":
    run_training()



from pathlib import Path
from typing import Dict, List, Optional

import yaml
from pydantic import BaseModel

import insurance_claim_model

PACKAGE_ROOT = Path(insurance_claim_model.__file__).resolve().parent
ROOT = PACKAGE_ROOT.parent
CONFIG_FILE_PATH = PACKAGE_ROOT / "config.yml"
DATASET_DIR = PACKAGE_ROOT / "datasets"
TRAINED_MODEL_DIR = PACKAGE_ROOT / "trained_models"


class AppConfig(BaseModel):

    package_name: str
    training_data_file: str
    testing_data_file: str
    pipeline_save_file: str


class ModelConfig(BaseModel):

    categorical_na: List[str]
    numerical_na: List[str]
    categorical_vars: List[str]
    numerical_vars: List[str]
    features: List[str]
    drop_columns: List[str]
    target: str
    test_size: float
    random_state: int
    n_estimators: int
    max_depth: int


class Config(BaseModel):
    app_config: AppConfig
    model_config: ModelConfig


def find_config_file() -> Path:

    if CONFIG_FILE_PATH.is_file():
        return CONFIG_FILE_PATH

    raise Exception(f"Config file is not at this path: {CONFIG_FILE_PATH}")


def fetch_config_file(cfg_path: Optional[Path] = None) -> Dict:

    if not cfg_path:
        cfg_path = find_config_file()

    if cfg_path:
        with open(cfg_path) as conf_file:
            config = yaml.safe_load(conf_file)
        return config

    raise Exception(f"Config file is not at this path: {CONFIG_FILE_PATH}")


def create_and_validate_config(parsed_config: Optional[Dict] = None) -> Config:

    if parsed_config is None:
        parsed_config = fetch_config_file(CONFIG_FILE_PATH)

    _config = Config(
        app_config=AppConfig(**parsed_config), model_config=ModelConfig(**parsed_config)
    )

    return _config


config = create_and_validate_config()



from typing import Union

import numpy as np
import pandas as pd

from insurance_claim_model import __version__
from insurance_claim_model.config.core import TRAINED_MODEL_DIR, config
from insurance_claim_model.processing.data_manager import load_pipeline
from insurance_claim_model.processing.validation import validate_input

pipeline_path = (
    f"{TRAINED_MODEL_DIR}/{config.app_config.pipeline_save_file}-{__version__}.pkl"
)
predict_pipeline = load_pipeline(pipeline_save_file=pipeline_path)


def predict_data(input_data: Union[pd.DataFrame, dict]) -> dict:

    input_data = pd.DataFrame(input_data)
    validated_data, errors = validate_input(input_data=input_data)
    results = {"prediction": None, "version": __version__, "errors": errors}

    if not errors:
        predictions = predict_pipeline.predict(validated_data)

        results = {
            "predictions": [np.exp(prediction) for prediction in predictions],  # type: ignore
            "version": __version__,
            "errors": errors,
        }

    return results



import os
from pathlib import Path
from typing import List

import joblib
import pandas as pd
from sklearn.pipeline import Pipeline

from insurance_claim_model import __version__
from insurance_claim_model.config.core import DATASET_DIR, TRAINED_MODEL_DIR, config


def load_dataset(*, file_name: str) -> pd.DataFrame:

    data = pd.read_csv(Path(f"{DATASET_DIR}/{file_name}"))
    data.drop(["index", "PatientID"], axis=1, inplace=True)

    return data


def save_pipeline(*, pipeline: Pipeline) -> None:

    pipeline_name = f"{config.app_config.pipeline_save_file}-{__version__}.pkl"
    save_path = TRAINED_MODEL_DIR / pipeline_name

    remove_old_pipeline(files_to_keep=[pipeline_name])
    joblib.dump(pipeline, save_path)


def load_pipeline(*, pipeline_save_file: str) -> Pipeline:

    pipeline_path = TRAINED_MODEL_DIR / pipeline_save_file
    train_model = joblib.load(pipeline_path)

    return train_model


def remove_old_pipeline(*, files_to_keep: List[str]) -> None:

    files_to_keep = files_to_keep + ["__init__.py"]
    for file in os.listdir(TRAINED_MODEL_DIR):
        if file not in files_to_keep:
            os.unlink(Path(TRAINED_MODEL_DIR / file))



from typing import Dict, List, Optional, Tuple, Union

import numpy as np
import pandas as pd
from pydantic import BaseModel, ValidationError

from insurance_claim_model.config.core import config


def drop_na_inputs(*, input_data: pd.DataFrame) -> pd.DataFrame:
    data = input_data.copy()
    new_na_columns = [
        var
        for var in data.columns
        if var not in config.model_config.categorical_na
        and var not in config.model_config.numerical_na
        and data[var].isnull().sum() > 0
    ]

    data.dropna(subset=new_na_columns, inplace=True)

    return data


def validate_input(*, input_data: pd.DataFrame) -> Tuple[pd.DataFrame, Optional[Dict]]:
    validated_data = drop_na_inputs(input_data=input_data[config.model_config.features])
    errors = None

    try:
        MultipleInsuranceInputs(
            inputs=validated_data.replace({np.nan, None}).to_dict(orient="records")
        )
    except ValidationError as error:
        errors = error.json()

    return validated_data, errors


class InsuranceDataInputScheme(BaseModel):
    index: Optional[int]
    PatientID: Optional[int]
    age: Optional[Union[float, int]]
    gender: Optional[str]
    bmi: Optional[float]
    bloodpressure: Optional[int]
    diabetic: Optional[str]
    children: Optional[int]
    smoker: Optional[str]
    region: Optional[str]
    claim: Optional[float]


class MultipleInsuranceInputs(BaseModel):
    inputs: List[InsuranceDataInputScheme]



from feature_engine.encoding import OneHotEncoder
from feature_engine.imputation import CategoricalImputer, MeanMedianImputer
from sklearn.ensemble import RandomForestRegressor
from sklearn.pipeline import Pipeline
from sklearn.preprocessing import MinMaxScaler

from insurance_claim_model.config.core import config

predict_pipe = Pipeline(
    [
        (
            "frequent_imputation",
            CategoricalImputer(
                imputation_method="frequent",
                variables=config.model_config.categorical_na,
            ),
        ),
        (
            "mean_imputation",
            MeanMedianImputer(
                imputation_method="mean", variables=config.model_config.numerical_na
            ),
        ),
        (
            "categorical_encoder",
            OneHotEncoder(
                drop_last=True, variables=config.model_config.categorical_vars
            ),
        ),
        ("scaler", MinMaxScaler()),
        (
            "RandomForest",
            RandomForestRegressor(
                random_state=config.model_config.random_state,
                n_estimators=config.model_config.n_estimators,
                max_depth=config.model_config.max_depth,
            ),
        ),
    ]
)
